#!/usr/bin/perl

use Test::More 'no_plan';

use FindBin qw($Bin);
use lib "$FindBin::Bin/lib", "$FindBin::Bin/../lib", "$FindBin::Bin/../etc";
use strict;

my $rc;

chdir("$Bin/..");

$rc = system("bin/rc.opsview stop");
is( $rc, 0, "Nagios stopped" );

# Check a db install works
system("bin/db_opsview db_install");
is( $?, 0, "Return code from db_opsview db_install is okay" );

# Confirm import/initial_opsview.sql is up to date with schema changes
system("bin/db_opsview db_export_initial > /tmp/initial_opsview.sql");
is( $?, 0, "db_opsview db_export_initial ok" );
my $diff = "diff -u import/initial_opsview.sql /tmp/initial_opsview.sql";
system("$diff > /dev/null");
if ( $? == 0 ) {
    pass("import/initial_opsview.sql as expected");
    unlink "/tmp/initial_opsview.sql" or die "Cannot remove /tmp/initial_opsview.sql: $!";
}
else {
    fail("Initial db config discrepency!!!\nTest with: $diff\nCopy with: cp /tmp/initial_opsview.sql $Bin/../import/initial_opsview.sql");
    if ( $ENV{OPSVIEW_TEST_HUDSON} ) {
        system("$diff");
    }
}

$rc = system("OPSVIEW_NOSTART=true bin/rc.opsview gen_config");
is( $rc, 0, "Config generated okay from a fresh install" );

$rc = system("bin/rc.opsview status") >> 8;
is( $rc, 1, "Nagios stays down because of OPSVIEW_NOSTART" );

# Allow time for opsview backup to complete
sleep 10;

# Delete all databases first, simulating a new install
# get configured database named, just in case they have been changed
use_ok('Opsview::Test::Cfg');
my $command = 'echo "';
for my $db (qw/ opsview runtime odw reports /) {
    $command .= 'DROP DATABASE IF EXISTS ' . Opsview::Test::Cfg->$db . '; ';
}
$command .= '" | mysql -u root';

#my $opsview = Opsview::Test::Cfg->opsview;
#my $runtime = Opsview::Test::Cfg->runtime;
#my $odw     = Opsview::Test::Cfg->odw;
#my $reports = Opsview::Test::Cfg->reports;
#
#my $command = "echo 'DROP DATABASE IF EXISTS $opsview; DROP DATABASE IF EXISTS $runtime; DROP DATABASE IF EXISTS $odw; DROP DATABASE IF EXISTS $reports' | mysql -u root ";
diag( 'About to run: ', $command ) if ( $ENV{TEST_VERBOSE} );
$rc = system($command);
is( $rc, 0, 'Previous databases dropped OK' );

my $connections;
foreach my $db (qw(runtime odw opsview reports)) {
    my $rc = system("bin/db_$db -t db_install");
    ok( $rc == 0, "Created $db database" );

    my $Db = ucfirst($db);

    my $dbh = eval "require $Db; $Db->db_Main";
    ok( defined $dbh, "Connected to $Db" );

    $connections->{$db} = $dbh;
}

is( $connections->{"opsview"}->selectrow_array("SELECT name FROM servicegroups WHERE id=1"), "Application - Opsview", "Must have servicegroup 1 = Application - Opsview" );
is( $connections->{"opsview"}->selectrow_array("SELECT name FROM servicegroups WHERE id=2"), "Network - SNMP MIB-II", "Must have servicegroup 2 = Network - SNMP MIB-II" );

is( $connections->{"opsview"}->selectrow_array("SELECT uuid FROM systempreferences"), "", "UUID is empty" );

is( $connections->{"opsview"}->selectrow_array("SELECT COUNT(*) FROM servicechecks WHERE uncommitted=1"), 0, "No uncommitted servicechecks" );
is( $connections->{"opsview"}->selectrow_array("SELECT COUNT(*) FROM hosttemplates WHERE uncommitted=1"), 0, "No uncommitted hosttemplates" );

is( $connections->{"opsview"}->selectrow_array("SELECT COUNT(*) FROM contacts WHERE show_welcome_page=0"), 0, "Welcome page to be shown to everyone" );

my @output = `installer/upgradedb.pl`;
my $num = grep /already up to date/, @output;
is( $num, 4, "Test if new installs cause an upgrade - if fails, probably db_* not updated with right version number" ) || print "Output:\n@output\n";

use_ok("Opsview::Test");

{
    local $/ = undef;
    my $output = `installer/upgradedb.pl`;
    unlike( $output, "/version/", "Some upgrade still required - if fails, probably not updated test dbs" );
}
