#!/usr/bin/perl
# Test NDOLogsImporter

use Test::More qw(no_plan);

use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib", "$Bin/../etc";
use Opsview::Utils::NDOLogsImporter;
use Test::MockObject;

my $break   = 0;
my $message = "";
my $level   = "";
my $mock    = Test::MockObject->new;
$mock->mock( "debug", sub { shift; $message = shift; $level = "debug"; } );
$mock->mock( "info",  sub { shift; $message = shift; $level = "info"; } );
$mock->mock( "error", sub { shift; $message = shift; $level = "error"; } );
$mock->mock( "warn",  sub { shift; $message = shift; $level = "warn"; $break++ if $message eq "Retrying attempt 3"; } );

my $importer = Opsview::Utils::NDOLogsImporter->new( { logger => $mock, break => \$break } );

is( $message, "", );
is( $level,   "" );
$importer->send_log("$Bin/var/empty");
is( $message,                  "Ignore $Bin/var/empty as size 0" );
is( $level,                    "info" );
is( $importer->last_file_size, 0 );

# Use our dummy file2sock
$importer->{file2sock} = "$Bin/bin/file2sock.dummy";

$importer->{ndo_socket} = "normal";
$importer->send_log("$Bin/var/ndolog");
is( $message,                  "Importing $Bin/var/ndolog. Size=1" );
is( $level,                    "debug" );
is( $importer->last_file_size, 1 );

$importer->{ndo_socket} = "other_error";
$importer->send_log("$Bin/var/ndolog");
like( $message, qr/file2sock command '.*' failed: rc=1, output=other_error/ );
is( $level, "error" );

$importer->{ndo_socket} = "connection_refused";
eval { $importer->send_log("$Bin/var/ndolog"); };
is( $message, "Retrying attempt 3" );
is( $level,   "warn" );
